import cv2

from mower.solvers.rogue import data
from mower.utils import config
from mower.utils.image import cropimg, diff_ratio, horizontal_stitch
from mower.utils.log import logger
from mower.utils.solver import BaseSolver


class WholeScene(BaseSolver):
    def stitch_images(self, images: list) -> cv2.Mat:
        try:
            stitcher = cv2.Stitcher.create(mode=cv2.Stitcher_SCANS)
            status, result = stitcher.stitch(images)
            if status == cv2.Stitcher_OK:
                logger.info("拼接完成。")
                return result
            else:
                logger.warning(f"拼接失败，状态码: {status}")
                raise RuntimeError(f"拼接失败，状态码: {status}")
        except cv2.error as e:
            logger.error(f"OpenCV 错误: {e}")
            raise
        except Exception as e:
            logger.error(f"拼接过程中出现错误: {e}")
            raise

    def get_whole_scene(self, draw=False):
        while True:
            img1 = config.recog.gray
            self.swipe_ext([(100, 870), (1920, 870)], [200], 400, 0.1)
            img2 = config.recog.gray
            if not diff_ratio(img1, img2, thresh=10, ratio=0.2):
                logger.info("滑到头了")
                break
        images = []
        new = cropimg(config.recog.img, ((0, 150), (1920, 880)))
        images.append(new)
        retry_time = 1
        while True:
            self.swipe_ext([(1820, 870), (1000, 870)], [200], 400, 0.1)
            new = cropimg(config.recog.img, ((0, 150), (1920, 880)))
            gray1 = cv2.cvtColor(new, cv2.COLOR_RGB2GRAY)
            gray2 = cv2.cvtColor(images[-1], cv2.COLOR_RGB2GRAY)
            flag = diff_ratio(gray1, gray2, thresh=10, ratio=0.2)
            if flag:
                images.append(new)
                retry_time = 1
            elif retry_time:
                retry_time -= 1
            else:
                logger.info("滑到底了")
                break
        logger.info(f"截图读取完了,有{len(images)}张截图")
        if len(images) > 1:
            stitch_img = horizontal_stitch(images)
        else:
            stitch_img = images[0]
        data.scene_image = cv2.GaussianBlur(stitch_img, (5, 5), 0)
        if draw:
            from matplotlib import pyplot as plt

            for img in images:
                plt.imshow(img)
                plt.show()
            plt.imshow(data.scene_image)
            plt.show()
